-- Only load once
if WRL and WRL.loading then goto EOF end

----------------------------------------
-- Environment management routines
----------------------------------------

WRL.Env = {}

-- Local functions
local _Gipairs = _G.ipairs
local _Gsprite = _G.sprite

-- Creates an environment table for the project, including dependencies
function WRL.Env.createInternal(meta)
    
    -- Get the asset root of the project
    local assetRoot
    if meta.version == "__LOCAL__" then -- Local project
        assetRoot = asset.documents[meta.name .. ".codea"]
    else -- Remote project
        assetRoot = asset.documents.webrepolib_cache[meta.name][meta.version][meta.name .. ".codea"]
    end
    
    local aorig = {
        documents = asset.documents,
        builtin = asset.builtin
    }
    
    -- New asset root table
    local newAsset = setmetatable({}, {
        __index = function(t, k)
            return aorig[k] or assetRoot[k]
        end,
        __concat = function(t, v)
            return assetRoot .. v
        end,
        __tostring = function() return tostring(assetRoot) end
    })
    
    -- Our new environment
    local env = {
        asset = newAsset,
        sprite = function(img, ...)
            if type(img) == "string" then
                local localAsset = img:match("Project:(.*)")
                if localAsset then
                    img = newAsset[localAsset]
                end
            end
            _Gsprite(img, ...)
        end
    }
    
    local depenvs = {} -- Blank dependent envs table.
    -- Passthrough to _G for read access if depenvs doesn't contain the value
    setmetatable(env, {
        __index = function(t, k)
            for _,env in _Gipairs(depenvs) do
                local v = env[k]
                if v then return v end
            end
            return _G[k]
        end
    })
    
    -- Return the env & the depenvs table so we can include deps
    -- after the fact
    return env, depenvs
end

-- Copies internal env members to new table, ignoring the metatable
-- in use for internal envs.
function WRL.Env.createExternal(internalEnv)
    local envExtern = {}
    for k,v in pairs(internalEnv) do
        envExtern[k] = v
    end
    return envExtern
end

-- Copies members of one environment table to another.
function WRL.Env.import(from, to)
    to = to or _ENV
    for k,v in pairs(from) do
        to[k] = v
    end
end

::EOF::