-- Only load once
if WRL and WRL.loading then goto EOF end

-- Check all current project source files for load guards on the first lines
if asset.name ~= "WebRepoLib.codea" then  -- exluding WRL itself.
    
    -- Read plist
    local f = io.open((asset .. "Info.plist").path, "r")
    local plist = f:read("*a") f:close()
    ppath = asset.path .. "/"
    
    local guardCode = 
[[----------------- DO NOT REMOVE OR MOVE -----------------
-- Ensure Codea doesn't load this file automatically
-- This MUST be at the top of this file!
if WRL and not WRL.loading then return end
--------------- END DO NOT REMOVE OR MOVE ---------------
]]
    local guardCodePat = guardCode:gsub("%-", "%%-"):gsub("%(%)", "%%(%%)")
    
    local restart = false
    
    -- Check each source file
    local buffers = plist:match("<key>Buffer Order</key>%s*<array>(.-)</array>")
    for b in buffers:gmatch("<string>(.-)</string>") do

        -- Don't add to the _dep_Documents
        if b ~= "_dep_Documents:WebRepoLib" then
    
            -- Read source file
            f = io.open(ppath .. b .. ".lua", "r")
            local src = f:read("*a")
            f:close()
                
            -- Check for load guard
            if src:match("^" .. guardCodePat) == nil then
                -- Add load guard if not present and request restart
                f = io.open(ppath .. b .. ".lua", "w")
                f:write(guardCode .. src)
                f:close()
                restart = true
            end
        end
    end
    
    if restart then
        -- ****************************************************
        -- WebRepoLib automatically adds load guards at the
        -- beginning of project source files to ensure it
        -- controls the load order of files and dependencies.
        --
        -- When these guards are added, we need to close and
        -- re-open the project editor to ensure Codea can see
        -- the changes!
        error("Close & Re-open your project!")
        -- ****************************************************
    end
end

::EOF::