-- Only load once
if WRL and WRL.loading then goto EOF end

local TaskManager = class()
local TaskBatch = class()

function TaskManager:init()
    self.batches = {}
    
    -- Init execution loop
    local scheduled = false
    local loop = function()
        scheduled = false
        local batch = self.batches[1]
        while batch ~= nil do
            self.currentBatch = batch
            -- Does this batch want to continue running?
            if batch:run() == true then
                self.scheduleLoop()
                break
            else
                table.remove(self.batches, 1)
                batch = self.batches[1]
            end
        end
        self.currentBatch = batch
    end
    
    self.scheduleLoop = function()
        if not scheduled then
            tween.delay(0.00001, loop)
            scheduled = true
        end
    end
end

function TaskManager:add(fn, ...)
    local batch = self.currentBatch
    if batch == nil then
        batch = TaskBatch()
        self:addBatch(batch)
    end
    batch:add(fn, ...)
    return batch
end

function TaskManager:addBatch(batch)
    table.insert(self.batches, batch)
    self.scheduleLoop()
end

function TaskManager:gate(condFn)
    local batch = self.currentBatch
    if batch == nil then
        batch = TaskBatch()
        self:addBatch(batch)
    end
    batch:gate(condFn)
    return batch
end

function TaskBatch:init(nonSeq)
    self.tasks = {}
    self.nonSeq = nonSeq or false
end

function TaskBatch:run()
    while true do
        local task = table.remove(self.tasks)
        if task == nil then
            return false -- The batch is done, we can move onto a new batch
        end
        
        local s, status = pcall(task.fn, table.unpack(task.args))
        if s then
            if status == false or self.nonSeq then -- Task has requested a yield
                return (#self.tasks > 0) -- Only continue the batch if we have tasks left
            end
        else
            -- TODO: TEST THIS!!!!!!!!
            -- We may need to delete all active batches instead
            self.tasks = {} -- Don't run any more tasks
            WRL.CrashHandler(status)
        end
    end
end

function TaskBatch:add(fn, ...)
    table.insert(self.tasks, { fn = fn, args = table.pack(...) })
end

function TaskBatch:gate(condFn)
    local task = nil
    
    task = function()
        if condFn() then return true end
        self:add(task)
        return false
    end
    
    if condFn() then return true end
    self:add(task)
end

WRL.TaskManager = TaskManager()
WRL.TaskBatch = TaskBatch

::EOF::